@@ -69,6 +69,28 @@ def idgen_seq():
     """
     return str(idgen_seq_counter.next())
 
+def err_response(reqid, code, msg, data=None):
+    """
+    Formats a JSON-RPC error as a dict with keys: 'jsonrpc', 'id', 'error'
+    """
+    err = { "code": code, "message": msg }
+    if data:
+        err["data"] = data
+    return { "jsonrpc": "2.0", "id": reqid, "error": err }
+
+def safe_get(d, key, def_val=None):
+    """
+    Helper function to fetch value from a dictionary
+    
+    * `d` - Dictionary to fetch value from
+    * `key` - Key to lookup in dictionary
+    * `def_val` - Default value to return if dict does not have a member with key
+    """
+    if d.has_key(key):
+        return d[key]
+    else:
+        return def_val
+
 class RpcException(Exception, json.JSONEncoder):
     """
     Represents a JSON-RPC style exception.  Server implementations should raise this
@@ -88,7 +110,7 @@ class RpcException(Exception, json.JSONEncoder):
             Optional extra info about the error. Should be a string, int, or list or dict of strings/ints
         """
         self.code = code
-        self.msg = msg
+        self.msg  = msg
         self.data = data
 
     def __str__(self):
@@ -97,6 +119,91 @@ class RpcException(Exception, json.JSONEncoder):
             s += "%s data=%s" % (s, str(self.data))
         return s
 
+class RequestContext(object):
+    """
+    Stores state about a single request, including properties passed
+    into Server.call
+    """
+
+    def __init__(self, props, req):
+        """
+        Creates a new RequestContext
+
+        :Parameters:
+          props
+            Dict of meta properties for this request
+          req
+            Dict that represents a single JSON-RPC request
+        """
+        self.props    = props
+        self.request  = req
+        self.response = None
+        self.error    = None
+
+    def func_name(self):
+        return unpack_method(self.request["method"])[1]
+
+    def get_prop(self, key, default_val=None):
+        """
+        Returns a property set on the context.
+
+        :Parameters:
+          key
+            String key to lookup in the context props dict
+          default_val
+            Value to return if key is not set on the context props
+        """
+        if self.props.has_key(key):
+            return self.props[key]
+        else:
+            return default_val
+
+    def set_error(self, code, msg, data=None):
+        """
+        Set an error on this request, which will prevent request execution.
+        Should only be called from "pre" hook methods.  If called from a post hook, this
+        operation will be ignored.
+
+        :Parameters:
+          code
+            Integer error code
+          msg
+            String description of the error
+          data
+            Optional additional info about the error. Should be a primitive, or a list or
+            dict of primitives to avoid serialization issues.
+        """
+        self.error = err_response(self.request["id"], code, msg, data)
+
+class Filter(object):
+    """
+    Base filter class that implements pre and post functions, but no-ops for both.
+    Subclass this and override pre/post to add filter functionality for your app.
+    """
+    
+    def pre(self, context):
+        """
+        Pre-Handler hook.  Called before the RPC request handler is invoked.
+        If context.send_error is called by any pre filter, then the request handler will
+        not be invoked, and the error will be returned instead.
+
+        :Parameters:
+          context
+            RequestContext instance for this request
+        """
+        pass
+
+    def post(self, context):
+        """
+        Post-Handler hook.  Called after the RPC request handler is invoked.
+        Post handlers can inspect and log the response, but should not alter it.
+
+        :Parameters:
+          context
+            RequestContext instance for this request
+        """
+        pass
+
 class Server(object):
     """
     Dispatches requests to user created handler classes based on method name.
@@ -123,6 +230,7 @@ class Server(object):
         self.validate_resp = validate_response
         self.contract = contract
         self.handlers = { }
+        self.filters = None
 
     def add_handler(self, iface_name, handler):
         """
@@ -140,7 +248,21 @@ class Server(object):
         else:
             raise RpcException(ERR_INVALID_REQ, "Unknown interface: '%s'", iface_name)
 
-    def call_json(self, req_json):
+    def set_filters(self, filters):
+        """
+        Sets the filters for the server.
+
+        :Parameters:
+          filters
+            List of filters to set on this server, or None to remove all filters.
+            Elements in list should subclass Filter
+        """
+        if filters == None or isinstance(filters, (tuple, list)):
+            self.filters = filters
+        else:
+            self.filters = [ filters ]
+
+    def call_json(self, req_json, props=None):
         """
         Deserializes req_json as JSON, invokes self.call(), and serializes result to JSON.
         Returns JSON encoded string.
@@ -148,15 +270,18 @@ class Server(object):
         :Parameters:
           req_json
             JSON-RPC request serialized as JSON string
+          props
+            Application defined properties to set on RequestContext for use with filters. 
+            For example: authentication headers.  Must be a dict.
         """
         try:
             req = json.loads(req_json)
         except:
             msg = "Unable to parse JSON: %s" % req_json
-            return json.dumps(self._err(None, -32700, msg))
-        return json.dumps(self.call(req))
+            return json.dumps(err_response(None, -32700, msg))
+        return json.dumps(self.call(req, props))
 
-    def call(self, req):
+    def call(self, req, props=None):
         """
         Executes a Barrister request and returns a response.  If the request is a list, then the
         response will also be a list.  If the request is an empty list, a RpcException is raised.
@@ -164,28 +289,30 @@ class Server(object):
         :Parameters:
           req
             The request. Either a list of dicts, or a single dict.
+          props
+            Application defined properties to set on RequestContext for use with filters. 
+            For example: authentication headers.  Must be a dict.
         """
+        resp = None
+
+        if self.log.isEnabledFor(logging.DEBUG):
+            self.log.debug("Request: %s" % str(req))
+
         if isinstance(req, list):
             if len(req) < 1:
-                return self._err(None, ERR_INVALID_REQ, "Invalid Request. Empty batch.")
-
-            resp = [ ]
-            for r in req:
-                resp.append(self._call_and_format(r))
-            return resp
+                resp = err_response(None, ERR_INVALID_REQ, "Invalid Request. Empty batch.")
+            else:
+                resp = [ ]
+                for r in req:
+                    resp.append(self._call_and_format(r, props))
         else:
-            return self._call_and_format(req)
+            resp = self._call_and_format(req, props)
 
-    def _err(self, reqid, code, msg, data=None):
-        """
-        Formats a JSON-RPC error as a dict with keys: 'jsonrpc', 'id', 'error'
-        """
-        err = { "code": code, "message": msg }
-        if data:
-            err["data"] = data
-        return { "jsonrpc": "2.0", "id": reqid, "error": err }
+        if self.log.isEnabledFor(logging.DEBUG):
+            self.log.debug("Response: %s" % str(resp))
+        return resp
     
-    def _call_and_format(self, req):
+    def _call_and_format(self, req, props=None):
         """
         Invokes a single request against a handler using _call() and traps any errors,
         formatting them using _err().  If the request is successful it is wrapped in a 
@@ -194,26 +321,47 @@ class Server(object):
         :Parameters:
           req
             A single dict representing a single JSON-RPC request
+          props
+            Application defined properties to set on RequestContext for use with filters. 
+            For example: authentication headers.  Must be a dict.
         """
         if not isinstance(req, dict):
-            return self._err(None, ERR_INVALID_REQ, 
+            return err_response(None, ERR_INVALID_REQ, 
                              "Invalid Request. %s is not an object." % str(req))
 
         reqid = None
         if req.has_key("id"):
             reqid = req["id"]
 
+        if props == None:
+            props = { }
+        context = RequestContext(props, req)
+
+        if self.filters:
+            for f in self.filters:
+                f.pre(context)
+
+        if context.error:
+            return context.error
+
+        resp = None
         try:
-            resp = self._call(req)
-            return { "jsonrpc": "2.0", "id": reqid, "result": resp }
+            result = self._call(context)
+            resp = { "jsonrpc": "2.0", "id": reqid, "result": result }
         except RpcException as e:
-            return self._err(reqid, e.code, e.msg, e.data)
+            resp = err_response(reqid, e.code, e.msg, e.data)
         except:
             self.log.exception("Error processing request: %s" % str(req))
-            return self._err(reqid, ERR_UNKNOWN, "Server error. Check logs for details.")
+            resp = err_response(reqid, ERR_UNKNOWN, "Server error. Check logs for details.")
         
+        if self.filters:
+            context.response = resp
+            for f in self.filters:
+                f.post(context)
 
-    def _call(self, req):
+        return resp
+
+    def _call(self, context):
         """
         Executes a single request against a handler.  If the req.method == 'barrister-idl', the
         Contract IDL JSON structure is returned.  Otherwise the method is resolved to a handler
@@ -223,6 +371,7 @@ class Server(object):
           req
             A dict representing a valid JSON-RPC 2.0 request.  'method' must be provided.
         """
+        req = context.request
         if not req.has_key("method"):
             raise RpcException(ERR_INVALID_REQ, "Invalid Request. No 'method'.")
 
@@ -245,6 +394,10 @@ class Server(object):
                 if self.validate_req:
                     self.contract.validate_request(iface_name, func_name, params)
 
+                if hasattr(iface_impl, "barrister_pre"):
+                    pre_hook = getattr(iface_impl, "barrister_pre")
+                    pre_hook(context, params)
+
                 if params:
                     result = func(*params)
                 else:
@@ -298,7 +451,7 @@ class HttpTransport(object):
         """
         data = json.dumps(req)
         req = urllib2.Request(self.url, data, self.headers)
-        f = urllib2.urlopen(req)
+        f = self.opener.open(req)
         resp = f.read()
         f.close()
         return json.loads(resp)
@@ -369,6 +522,8 @@ class Client(object):
             for logging or other purposes.  UUIDs are used by default, but you can substitute another
             function if you prefer something shorter.
         """
+        logging.basicConfig()
+        self.log = logging.getLogger("barrister")
         self.transport = transport
         self.validate_req  = validate_request
         self.validate_resp = validate_response
@@ -379,6 +534,12 @@ class Client(object):
         for k, v in self.contract.interfaces.items():
             setattr(self, k, InterfaceClientProxy(self, v))
 
+    def get_meta(self):
+        """
+        Returns the dict of metadata from the Contract
+        """
+        return self.contract.meta
+
     def call(self, iface_name, func_name, params):
         """
         Makes a single RPC request and returns the result.
@@ -392,7 +553,11 @@ class Client(object):
             List of parameters to pass to the function
         """
         req  = self.to_request(iface_name, func_name, params)
+        if self.log.isEnabledFor(logging.DEBUG):
+            self.log.debug("Request: %s" % str(req))
         resp = self.transport.request(req)
+        if self.log.isEnabledFor(logging.DEBUG):
+            self.log.debug("Response: %s" % str(resp))
         return self.to_result(iface_name, func_name, resp)
 
     def to_request(self, iface_name, func_name, params):
@@ -535,80 +700,58 @@ class Batch(object):
 
     def send(self):
         """
-        Sends the batch request to the server and returns a BatchResult instance.
-        Raises a RpcException if the batch has already been sent.
+        Sends the batch request to the server and returns a list of RpcResponse
+        objects.  The list will be in the order that the requests were made to
+        the batch.  Note that the RpcResponse objects may contain an error or a 
+        successful result.  When you iterate through the list, you must test for
+        response.error.
+        
+        send() may not be called more than once.
         """
         if self.sent:
             raise Exception("Batch already sent. Cannot send() again.")
         else:
-            resp = self.client.transport.request(self.req_list)
             self.sent = True
-            return BatchResult(self, self.req_list, resp)
+            results = self.client.transport.request(self.req_list)
+            
+            id_to_method = { }
+            by_id = { }
+            for res in results:
+                reqid = res["id"]
+                by_id[reqid] = res
 
-class BatchResult(object):
+            in_req_order = [ ]
+            for req in self.req_list:
+                reqid  = req["id"]
+                result = None
+                error  = None
+                resp   = safe_get(by_id, reqid)
+                if resp == None:
+                    msg = "Batch response missing result for request id: %s" % reqid
+                    error = RpcException(ERR_INVALID_RESP, msg)
+                else:
+                    r_err = safe_get(resp, "error")
+                    if r_err == None:
+                        result = resp["result"]
+                    else:
+                        error = RpcException(r_err["code"], r_err["message"], safe_get(r_err, "data"))
+                in_req_order.append(RpcResponse(req, result, error))
+            return in_req_order
+                
+
+class RpcResponse(object):
     """
-    Holds the results from a Batch.send() call.  Individual results are unmarshaled during each
-    call to get()
+    Represents a single response in a batch call.  Has the following properties:
+    
+    * `request` - JSON-RPC request dict
+    * `result`  - Result from this call. Set to None if there was an error.
+    * `error`   - RpcException instance.  Set to None if call was successful.
     """
-
-    def __init__(self, batch, req_list, resp):
-        """
-        Creates a new BatchResult, automatically reordering to the response to match the order of
-        the requests.
-
-        Raises RpcException if the resp length does not match the req_list length, or if any
-        element in resp cannot be correlated to req_list based on id.
-
-        Use the "count" property on the BatchResult instance to get the length of the response.
-
-        :Parameters:
-          batch
-            Batch instance associated with this result
-          req_list
-            List of requests that were sent in this batch in (list of dicts in JSON-RPC request format)
-          resp
-            List of results from the server (lists of dicts in JSON-RPC response format)
-        """
-        if len(req_list) != len(resp):
-            msg = "Batch response length %d != request %d" % (len(resp), len(req_list))
-            raise RpcException(ERR_INVALID_RESP, msg)
-
-        self.id_to_method = { }
-        by_id = { }
-        for r in resp:
-            reqid = r["id"]
-            by_id[reqid] = r
-
-        in_req_order = [ ]
-        for r in req_list:
-            reqid = r["id"]
-            if not by_id.has_key(reqid):
-                msg = "Batch response missing result for request id: %s" % reqid
-                raise RpcException(ERR_INVALID_RESP, msg)
-            in_req_order.append(by_id[reqid])
-            self.id_to_method[reqid] = r["method"]
-
-        self.batch = batch
-        self.resp = in_req_order
-        self.count = len(in_req_order)
-
-    def get(self, i):
-        """
-        Returns a single response from the BatchResult based on offset.  If the offset is out of range
-        an IndexError is raised.  If the response contained an error, a RpcException is raised with the
-        error information for that response.
-
-        :Parameters:
-          i 
-            Offset into the result, zero based.
-        """
-        if i < self.count:
-            resp = self.resp[i]
-            method = self.id_to_method[resp["id"]]
-            iface_name, func_name = unpack_method(method)
-            return self.batch.client.to_result(iface_name, func_name, resp)
-        else:
-            raise IndexError("%d >= result size: %d" % (i, self.count))
+    
+    def __init__(self, request, result, error):
+        self.request = request
+        self.result  = result
+        self.error   = error
 
 class Contract(object):
     """
@@ -627,6 +770,7 @@ class Contract(object):
         self.interfaces = { }
         self.structs = { }
         self.enums = { }
+        self.meta = { }
         for e in idl_parsed:
             if e["type"] == "struct":
                 self.structs[e["name"]] = Struct(e, self)
@@ -634,6 +778,10 @@ class Contract(object):
                 self.enums[e["name"]] = Enum(e)
             elif e["type"] == "interface":
                 self.interfaces[e["name"]] = Interface(e, self)
+            elif e["type"] == "meta":
+                for k,v in e.items():
+                    if k != "type":
+                        self.meta[k] = v
 
     def validate_request(self, iface_name, func_name, params):
         """
@@ -904,12 +1052,28 @@ class Struct(object):
             else:
                 return False, "field '%s' not found in struct %s" % (k, self.name)
 
-        for k, v in self.fields.items():
-            if not val.has_key(k) and not v.optional:
-                return False, "field '%s' missing from: %s" % (k, str(val))
+        all_fields = self.get_all_fields([])
+        for field in all_fields:
+            if not val.has_key(field.name) and not field.optional:
+                return False, "field '%s' missing from: %s" % (field.name, str(val))
 
         return True, None
 
+    def get_all_fields(self, arr):
+        """
+        Returns a list containing this struct's fields and all the fields of
+        its ancestors.  Used during validation.
+        """
+        for k, v in self.fields.items():
+            arr.append(v)
+            
+        if self.extends:
+            parent = self.contract.get(self.extends)
+            if parent:
+                return parent.get_all_fields(arr)
+
+        return arr
+
 class Function(object):
     """
     Represents a function defined on an Interface
